******************************************************************************************************
* Table Loader = TABLDR
* BAS and BIN are essentially the same - both start with an 9 byte header of the following format:
*    Offset:   Type:   Value:				  End-offset:
*      0       byte    $55     Constant	  -9
*      1       byte    <FILETYPE>			  -8
*      2:3     word    <LOAD>					  -7
*      4:5     word    <LENGTH>				  -5
*      6:7     word    <EXEC>					  -3
*      8       byte    $AA     Constant	  -1
*      9-xxx   byte[]  <DATA>
*
*    <FILETYPE> is
*        $01     for BAS
*        $02     for BIN
******************************************************************************************************
* DWRead
*    Receive a response from the DriveWire server.
*    Times out if serial port goes idle for more than 1.4 (0.7) seconds.
*    Serial data format:  1-8-N-1
*    4/12/2009 by Darren Atkinson
* Entry:
*    X  = starting address where data is to be stored
*    Y  = number of bytes expected
* Exit:
*    CC = carry set on framing error, Z set if all bytes received
*    X  = starting address of data received
*    Y  = checksum
*    U is preserved.  All accumulators are clobbered
**********************************************************************************
* DWWrite
*    Send a packet to the DriveWire server.
*    Serial data format:  1-8-N-1
*    4/12/2009 by Darren Atkinson
* Entry:
*    X  = starting address of data to send
*    Y  = number of bytes to send
* Exit:
*    X  = address of last byte sent + 1
*    Y  = 0
*    All others preserved
**********************************************************************************
* DoRead() Asks DW4 to send a whole sector
* X is the LSN number for this sector
* Y is the destination where the sector data has to be put
* uses DNUM as drive number
**********************************************************************************
*
       ORG   $2000
       PUT   $2000
*
noexec   equ   $10               * run/exec flag (don't do it if zero) 
dnum     equ   $12					* assigned drive number
numsec   equ   $76					* number of sectors in extent in process
tabptr 	equ   $f7					* pointer to table entries (2 bytes)
DWDVEC	equ   $e6					* DoRead vector (2 bytes)
sector	equ   $01cf					* next sector to be read (2 bytes)
nambuf   equ   $01d1					* fixed by BASIC read filename function which stores in $1d1
table    equ   $02E0					* sectors per extent Table			
newdes   equ   $0350					* new destination for part two
drghdr   equ   nambuf				* header will be copied here
strbuf 	equ   nambuf+8				* beginning of circular buffer
endbuf   equ   strbuf+256	 		* end of circular buffer
*
dimask   equ   $50					* mask to disable interrupts(FIRQ and IRQ)
eimask   equ   $af					* mask to enable interrupts
*
BASV2 	equ   $83ed					* initialize BASIC
BASV1 	equ   $841f					* reset BASIC stack etc
RUNBAS 	equ   $849f					* run_basic
OUTSTR   equ   $90e5					* print string at X+1 to DEVNUM
OUTCHR   equ   $B54A					* print char A to DEVNUM
*
       	ldx   ,s						* get stack pointer ro reg 'X'
       	ldu   5,X					* get 3rd address from vector table
       	stu   <DWDVEC				* save as vector for DWDOREAD
*       	
			ldx   #DWTEXT-1			* message table loader
			jsr   OUTSTR				* print string
entry		ldx   #table				* point to beginning of table
			lda   ,x+					* get drive number
			beq   MSG02             * show message: "No drive number" and quit
			sta   <dnum					* save drive number 
			lda   ,x+					* get run/exec flag
			sta	<noexec				* save run/exec flag
			ldd   ,x++					* get first LSN
			std   >sector				* put in working variable
			lda   ,x+					* get number of sectors in this extent
			sta   <numsec				* put in working variable
			stx   <tabptr				* save pointer to table
*
			orcc	#dimask				* disable interrupts to work with DW4
			ldu   #endbuf    			* point to end of buffer (start with empty buffer)
         ldy   #9						* read 9 bytes (Dragon header)
			ldx   #drghdr   			* header destination
			lbsr  cpybuf    			* moves x bytes ahead
										* verify the header we just read		
			lda   -9,x       			* get first data byte
			cmpa  #$55       			* Dragon DOS file?
			bne   MSG01					* show message: "Not a Dragon file" and quit
*			
        	pshs  d,x,u					* save registers
			ldx   #sndprt				* point to reallocatable part
			ldu   #newdes  			* point to new destination
realoc   ldd   ,x++					* get 2 bytes
			std   ,u++					* copy to destination
			cmpx  #ENDPGM				* got to end?
			bls   realoc				* no, keep on
         puls  d,x,u				* restre registers
         jmp   newdes 				* transfer control to reallocated part
*
MSG02    ldx   #NODRVN-1         * point to message No drive Number
			fcb   #$10              * to jump over next one (forming a ldy #$nnnn)
MSG01    ldx   #NOTDF-1				* point to message Not Dragon file
			jsr   OUTSTR				* show it to screen
         ldx   #$02dc				* beginning of interpreters buffer
			stx   <#$A6					* get nextchar points to it
			clra							* zero
			clrb							* 16 bits
			std   ,X						* clean buffer
      	andcc #eimask				* enable interrupts			
			rts							* return to the caller (and from there to the interpreter)			
*
DWTEXT	FCC   /FILE LOADER/,13,0* initial message
NOTDF    FCC   /NOT A DRAGON/
		   FCC   / FILE/,0			* not Dragon file
NODRVN   FCC   /NO DRIVE /
			FCC   /NUMBER/,0			* no drive number received

**********************************************************************************
* SECOND PART OF PROGRAM - TO BE REALLOCATED BELOW $0400
**********************************************************************************
sndprt	dec   -8,x       			* decrement file type (Dragon DOS BASIC is 1)
			bne   ddbin					* if not zero it was a 2, so binary file
ldbas		ldy   -5,x       			* get file length (read whole BASIC program)
			ldx   <$19					* get start of basic
			bsr   cpybuf				* read whole file ... sectors are in Table !
basptr	stx   <$1b       			* start for simple variables
			stx   <$1d					* start for array variables
			stx   <$1f					* end of storage in use
			jsr   BASV2    			* BasVect2 initialize BASIC
			tst   <noexec				* should it be run?
			beq   msgou2					* no, go out
			jsr   BASV1    			* BasVect1 reset BASIC stack etc
			jmp   RUNBAS				* run basic
*
ddbin		ldy   -5,x       			* file length					
			ldx   -7,x       			* load address
			bsr   cpybuf    			* read whole file ... sectors are in Table !
			ldx   #drghdr+6 			* point to exec address
			ldx   ,x       			* get exec address
			tfr   x,d					* transfer to D
			inca							* test address range
			beq   msgou2				* return to basic command loop if exec address $FFxx
			stx   <$9d       			* save BASIC EXEC address
			tst   <noexec				* should be executed?
			beq	msgou2				* no, don't do it
      	andcc #eimask				* enable interrupts			
			jsr   ,x						* exec it
			rts							* return
*			
MSG03    leas	6,s					* clean stack
			leax  NOMORE,PCR			* point to message End of Table
			jsr   OUTSTR				* show it to screen
msgou2   ldx   #$02dc				* beginning of interpreters buffer
			stx   <#$A6					* get nextchar points to it
			clra							* zero
			clrb							* 16 bits
			std   ,X						* clean buffer
      	andcc #eimask				* enable interrupts			
			rts							* return to the caller (and from there to the interpreter)
*
ErrDW    leas  6,s					* clean stack
			bra   msgou2				* back to the caller
*
**********************************************************************************
* Works with a circular buffer. U must point to the buffer
* Y is the required number of bytes to be read (will read a whole sector)
* X must point to the destination address where the Y bytes will be copied
**********************************************************************************
cpybuf   cmpu  #endbuf				* got to the end of buffer?
			bne   copym					* no, so copy data from buffer 
			pshs  x,y					* save parameters
			ldx   >sector				* get sector to read from
			cmpx  #$FFFF				* reached end of table?
			beq   MSG03					* no more data to read. Problem!
			ldy   #strbuf				* point to beginning of circular buffer
			tfr   y,u					* copy buffer pointer to reg 'U'
			jsr   (DWDVEC)				* ask DW4 to send a whole sector
			bcs   ErrDW 				* detect frame error
			bne   ErrDW 				* detect incomplete read operation
getsec	pshs  x						* save register X
			dec   <numsec				* decrement sectors in this extent
			ldb   <numsec				* get sectors left in this extent
			bne   nxtone				* if not zero, next one in that extent
nxtext   ldx   <tabptr				* get table pointer
         ldd	,x++					* get 1st LSN in next extent
         std   >sector				* put in variable
         lda   ,x+					* get number of sectors in this extent
         sta   <numsec				* put in variable
         stx   <tabptr				* update pointer
         bra   gets01				* out
nxtone   ldx	>sector				* get old LSN
			leax  1,x					* increment
			stx   >sector				* save next sector
gets01   puls  x						* restore X
			lda   #'.					* print dot
			jsr   OUTCHR            * to screen to show progress
			puls  x,y					* get parameters back
copym		lda   ,u+					* get a byte from buffer
			sta   ,x+					* copy at destination
			leay  -1,y					* decrement counter
			bne   cpybuf				* if not all done, keep on
NOMORE	rts							* return

***********************************************************************************	
         FCC   /END OF TABLE/    * reached end of Table			
ENDPGM   FCB   0,0					* end of String and of Program
*
