* =========================================================================================================================
* VLOAD / Load a file from inside a VDK file
* This dweeb will be loaded by DLOAD this way:
* DLOAD"VLOAD"[p]"DiskName.VDK""FileName.ext
* For the VDK-Disk, if '.' not found in the name, then .VDK is automatically added 
* The 'p' parameter can only be a "N" in order to not run the loaded program
* any other character will be ignored. If no parameter, then program will be executed/run
* =========================================================================================================================
* Technical notes:
* The buffer for K7 & RS232 is at $1DA - $2D8
* DLOAD uses in function GETNAM $1D1 for the name length and from $1D2 for the whole name
* The read buffer for DW4 is from $1DA to $2DA
* there are 5 bytes left until $2DF
* Assuming 4 extents in the file entry and four additional entry extensions (7 extents each) = 4 + 4*7 = 32 3 bytes-blocks
* this will need as much as 96 bytes plus drive number and noexec flag at the beginning and two EndOfTable bytes at the end
* equals 100 bytes($64)
* The sectors table will be then from $2E0 to $343:
* $2E0 will be the DW4 assigned drive number
* $2E1 will be the run/exec flag
* $2E2-$2E3 the first LSN in an extent  -  $2E4 the number of sectors in this extent
*     ... the same repeated as many times as needed ... for instance the very last one could be:
* $33F-$340 the first LSN in last entry -  $341 the number of sectors in last extent (hopefully never sooooo long)
* After the last extent data, an EndOfTable record should be put:
* $342-$343 = $FFFF
*
* Once we have filled the table, we need another program to ask DW4 to send exactly the sectors stored in the table
* The very first sector will contain the Dragon file header, whith the load and exec addresses.  
* In this case we don't need to deal with CoCo headers, then Dragon Data uses only its own one.
* We would have a string like: "TABLDR" being TABLDR the special program above mentioned
* The buffer for the interpreter begins at $2DC and this is pointed to from $A6-$A7 in the getNextChar Basic routine
* We will call VLOAD right from ROM at $A049, and $A6-A7 will point to the "TABLDR" text
* Despite this program will be loaded at $2000 it must leave as much free memory as possible, so it will copy a part of it 
* at $350 - $3FF from where it will be loading the needed sectors and run/exec it later
*
* =========================================================================================================================
*
       ORG   $2000
       PUT   $2000
*
NOEXEC EQU   $10							* inhibit exec after loading (0 = NO EXEC)
DNUM   EQU   $12							* assigned drive number by DW4 server
SEC1ST EQU   $76							* 1ST directory sector (2 bytes)
TABPTR EQU   $F7							* pointer to table entry (2 bytes)
SECDON EQU   $F9                    * number of directory sectors read
*
DIMASK EQU   $50							* mask to disable interrupts(FIRQ and IRQ)
EIMASK EQU   $AF							* mask to enable interrupts
NAMBUF EQU   $01D1						* fixed by BASIC, read filename function which stores in $1d1
TABLE  EQU   $02E0                  * Table with all needed sectors for the requested file
HEADER EQU   $2F00                  * place where header (sector zero) is to be loaded (2F00-2FFF)
DIRENT EQU   $3000						* where to copy the directory sector containing the entry in process
EXTENT EQU   $3100						* where to copy the directory sector with extensions for the entry in process
FILNAM EQU   $3200						* buffer for the 'standardized' filename read from entry
*
SYSERR EQU   $8344						* Show error message (B=error code)
OUTSTR EQU   $90E5						* print string at X+1 to DEVNUM
GETNAM EQU   $B7AA						* read file name and length into $1d1
IOERR  EQU   $B84B						* IO ERROR message
*
ENTRY  JSR   <$A5							* peek at next character (after "VDKLOAD")
       SUBA  #'N							* to detect if it was an 'N' - if so don't run/exec it
       STA   <NOEXEC 					* save difference. 0 = noexec
 		 BNE	 NOJUMP						* if not 'N' don't skip it
		 JSR   <$9F							* skip that 'N'
NOJUMP BSR   JMPCHR          			* see if there is an arguments separator
											* initial message and save DLOAD vectors
SAVVEC LBSR  MSG01 					   * print initial message
       LDX   ,S							* get stack pointer ro reg 'X'
       LDU   1,X							* get 1ST address from vector table
       STU   DWRVEC  					* save as vector for DWREAD
       LDU   3,X							* get 2nd address from vector table
       STU   DWWVEC						* save as vector for DWWRITE
       LDU   5,X							* get 3rd address from vector table
       STU   DWDVEC						* save as vector for DWDOREAD
											* get the VDK name (adding extension if not present)
GETN   ORCC  #DIMASK	            	* disable FIRQ, IRQ (to work with DW4)
		 JSR   GETNAM						* read VDK filename and length into $1d1 = nambuf
       LDX   #NAMBUF-1					* byte before packet buffer start
       CLR   ,X							* zero MSB to make a 16 bit value for Y
       LDY   ,X++							* length of file name (16 bit)
       BEQ   FAILED						* if len=0 no file entered
       LDB   -1,X							* length of name (8 bit)
											* Adding .VDK if not termination entered      
		 PSHS  B,X							* save pointer to beginning of name and name length
ISDOT  LDA   ,X+							* get a char from name
       CMPA  #'.							* is it a dot?
       BEQ   ALRDOT						* yes, stop search, quit loop
       DECB									* decrement counter
       BNE   ISDOT						* not end of string, get next char
       LDD   #$2E56						* get chars ".V"
       STD   ,X++							* add to end of name
       LDD   #$444B						* get chars "DK"
       STD   ,X++							* add to end of name
		 LDB   ,S							* get old name length
		 ADDB  #4							* add 4 (because of .VDK added)
		 STB   ,S							* update new length in stack
       STB   >NAMBUF						* save new length in byte before string
       LEAY  4,Y                    * correction to name length - DW parameter
ALRDOT PULS  B,X       					* restore pointer and length
		 CLR   B,X							* zero terminate name string
											* ask DW4 to mount the VDKname or fail
       INC   ,--X							* 1 = DriveWire OP_NAMEOBJ_MOUNT command
       LEAY  2,Y							* length of DW packet = name length + 2
       JSR   (DWWVEC)					* send file request to server
       LDX   #DNUM						* our drive number variable
       CLR   ,X							* clear it
       LEAY  1,Y							* read one byte back
       JSR   (DWRVEC)					* get assigned drive number
       TST   ,X							* verify received value
       BNE   VDHEAD						* if not zero, successful mount                                       
       LDB   #$19							* code for MO ERROR
ERROUT ANDCC #EIMASK						* enable interrupts
		 JMP   SYSERR						* show error message
											* Messages Dispatcher
JMPCHR JSR   <$A5							* gets last read char
       CMPA  #'"							* is it a double quote?
       BEQ   JMP01						* yes, don't jump over it
       JSR   <$9F							* no, skip it
JMP01  RTS									* return
*
MSG07  CLRB									* code for NF ERROR
		 BRA   ERROUT						* show error message
MSG06  LEAS	 2,S                    * to get rid of PC put in stack by MOREXT
		 LDX   #BADVDK-1              * point to message "Bad VDK directory"
		 FCB   #$10							* forms LDY #xxxx with next instruction, to jump it
MSG05  LDX   #NULFIL-1              * point to message "Null File"
		 FCB   #$10							* forms LDY #xxxx with next instruction, to jump it
MSG04  LDX   #NOTDW-1					* point to error message "Not DW-adapted file"
		 FCB   #$10							* forms LDY #xxxx with next instruction, to jump it
MSG03  LDX   #NOTVDK-1					* point to error message "Not VDK file"  
  		 FCB   #$10							* forms LDY #xxxx with next instruction, to jump it
MSG02  LDX   #NOTFIL-1					* point to error message "No FILE Entered"  
       ANDCC #EIMASK						* enable interrupts
  		 FCB   #$10							* forms LDY #xxxx with next instruction, to jump it
MSG01  LDX   #LDTEXT-1					* point to intro message
       JSR   OUTSTR						* print message to screen
		 RTS									* return
*
FAILED ANDCC #EIMASK						* enable interrupts
   	 JMP   IOERR						* I/O error message
*											* treats any char different from double quote as an arguments separator and skips it
   	 									* read the VDK header and verify it is OK
VDHEAD LDX   #0							* header sector to be read through DW4
       LDY   #HEADER						* point to header RAM area
       JSR   (DWDVEC)					* read header
       BCS   FAILED						* detect frame error
       BNE   FAILED						* detect incomplete read operation
       LDD   #$646B                 * first word (byte numbers 0-1) should be 'dk' (lowercase)
       CMPD  ,Y                     * is a VDK file?
       BNE   MSG03				      * no, show error (Not an VDK file)
		 LDB   2,Y  						* header length. Low 8 bits come in byte 2
		 LDA   3,Y                    * header length. High 8 bits come in byte 3
		 CMPD	 #$0100						* is it a DW-adapted VDK?
       BNE   MSG04						* no, show error (Not DW-adapted VDK)
											* calculate with header params the first directory sector to be read
VDK    LDX   #361							* 1st directoy sector for a 180k disk
		 LDB   9,Y							* byte number 9 is number of sides (1=single for 180K - 2=double for 360k and 720k upon byte 8)
		 CMPB  #2							* is double sided?
		 BNE   NOTX2						* no, don't double tracks
		 LDX	 #721							* 1st directory sector for double sided disks
NOTX2	 LEAX  2,X							* jump over the two first sectors of the directory track (FAT)
		 STX   <SEC1ST						* save number of 1st directory data entry sector
											* get the filename to be loaded/executed (will stay at $1D2 with length al $1D1)
 		 BSR   JMPCHR          			* see if there is an arguments separator
       JSR   GETNAM						* read filename to be loaded and length into $1d1 = nambuf
       LDB   >NAMBUF						* length of filename (8 bit)
       BEQ   MSG02						* if len=0 no file entered, show message and stop

****************************************************************************************************
* Directory entries Process: Searching the requested filename
****************************************************************************************************
		 CLR   <SECDON						* zero dir sectors read
		 LDX   #TABLE						* get beginning of the table at RAM
		 STX   <TABPTR						* save at working variable
		 CLRA									* zero
		 CLRB									* 16 bits
		 STD   ,X							* initialize table: drive 0, run flag equal true
		 STA	 4,X							* initialize table: numSects = $0
		 COMA									* convert into $FF
		 COMB									* convert into $FF
		 STD	 2,X							* initialize table: 1st LSN = $FFFF
		 LDX   <SEC1ST                * get first data directory sector (track 20 sector 3)
                             			* for each data directory sector (from SEC1ST to SEC1ST+15 = 3 to 18 track 20)
NXTSEC LDY   #DIRENT							* point to directory entries area in RAM
													* ask DW4 to load the sector
       JSR   (DWDVEC)						* read directory sector
       BCS   FAILED							* detect frame error
       BNE   FAILED							* detect incomplete read operation
													* for each one of the 10 file entries
														* verify that is a valid entry, else go to next entry
       LDU   #DIRENT								* point to read data area
NENT01 LDA   ,U									* read flags of this entry
       BITA  #8									* if Flags AND 8 = 8 then END of Directory reached
       BNE   MSG07								* if so, show error message: Not Found
       BITA  #128+1              	  		* if Flags AND 1 = 1 this is an Extension, don't use it and go to next entry
       												* or Flags AND 128 = 128 then is a Deleted File, forget about it and go to next entry
       LBNE  NXTENT									* if any of them go to next entry
														* got a valid file entry. Read FileName and Extension (carefull, both are right padded with nulls)
EXT01A LDX   #FILNAM             	   	* point to standardized filename buffer in this entry
		 LDD   #$0108								* offset 1-8 = FileName
    	 LBSR  SAVNAM								* put name into buffer (skipping nulls)
       LDA   #'.									* get dot ascii code
       STA   ,X+									* add to buffer
		 LDD   #$0903								* Offset 9-11 = Extension
	 	 LBSR  SAVNAM								* add to buffer (skipping nulls)
	 	 CLR   ,X									* put a null at the end
														* compare this standardized filename against the required one
		 LDX   #FILNAM								* point to the beginning of read filename
		 LDY   #NAMBUF								* point to the beginning of required file
 		 LDB   ,Y+									* get file length and point to 1st char
CMPSTR LDA   ,X+									* get one char from standardized filename
		 CMPA  ,Y+									* is the same than the one in required file?
		 BNE   NXTENT									* no, go to next file entry
		 DECB											* decrement counter
		 BNE   CMPSTR								* if more chars to compare, go back
														* coincidence found. Put DW4 assigned drive number at $200
SUCCES LDX   #TABLE								* point to data Table
       LDA   <DNUM								* get drive number
       STA   ,X+									* save it
       LDA   <NOEXEC								* get run/exec flag
       STA   ,X+									* add to table
										   	      * for each extent in that file entry (four)
EXT01  LDA   14,U										* is offset 14 zero (num sectors for 1st extent)?   
		 LBEQ	 MSG05										* NO sectors used, stop and show error message
		            									* add used sectors by this file
       LDB   #14										* offset to number of sectors in 1st extent
EXTLOP SUBB  #2										* offset to 1st sector LSN
		 LDY   B,U										* get LSN
    	 LEAY  1,Y										* because header occupies a whole sector in VDK!		 
		 STY   ,X++										* put LSN in table
		 STA   ,X+										* put numSects in table
		 ADDB  #5										* gives offset to number of sectors in next extent	 CMPB  #23										
		                                 * is it out of limits (past 4th extent)?
		 CMPB  #23										* is it out of limits (past 4th extent)?
		 BHI   EXT04A									* yes, out of this extents loop
		 LDA   B,U         							* is number of sectors in next extent equal to zero?
       BNE   EXTLOP										* no, back to process next extent
       BRA   CALNXT									* out to call next dweeb
		 											* see if there are more extents to analyze
EXT04A STX   <TABPTR								* save tabpointer
		 LDA	 ,U									* get Flags of this entry
       BITA  #32									* if Flags AND 32 = 32 there are more extents
       BEQ   CALNXT              	   	* else out of here to call next dweeb
 		 LBSR	 MOREXT 								   * call MoreExtents(Offset 24)
		 BRA   CALNXT								   * out to call next dweeb
*
NXTENT LEAU  25,U				 					* point to next entry - (step 25 bytes)
       CMPU  #DIRENT+250						* detected end of sector entries?
       BLO   NENT01							* no, process next file entry
       INC   <SECDON							* increment number of dir sectors processed
       LDB   <SECDON							* get this value
       CMPB  #16                 	   * finished whole directory? (from 3 to 18, so 16 data sectors)
       LBHS  MSG07							* yes, error message File Not Found
		 LDX   <SEC1ST		 					* get first directory sector
		 ABX										* add sector number to be processed
       LBRA	 NXTSEC						* process next directory sector
       
********************************************************************************************************* 
* Adds name or extension to the filename buffer pointed by X  -  U = points to sector entry
* A contains offset to beginning of text - B contains length of data to be copied
********************************************************************************************************* 
SAVNAM LEAY  A,U							* point to data from sector entry plus offset
ADD01  LDA   ,Y+							* get a byte
		 BEQ   NOADD						* if null, quit
       STA   ,X+							* add to buffer
		 DECB									* decrement counter
	    BNE   ADD01						* not all done, keep on
NOADD  RTS									* return

********************************************************************************************************* 
* call next dweeb if everything is alright
********************************************************************************************************* 
CALNXT LDD   #$FFFF						* end record mark: LSN number ($FFFF)  
		 STD   ,X++							* put at the end of table data
		 CLR   ,X+							* end record mark: sectors used ($00)
		 LDX   #NDWEEB						* get literal text for next dweeb
		 STX   <$A6							* point $A6-$A7 to the next dweeb ("TABL1")
	 	 LEAS  2,S							* clean 'old' DLOAD call from stack
		 ANDCC #EIMASK						* enable interrupts (if something goes wrong ... )
		 JMP   $A049						* calls DLOAD again with next needed dweeb
* 	 	 RTS									* return

*************************************************************************************
* Process extra extents (additional ones). Called with U pointing to old entry 
*************************************************************************************
MOREXT LDA   24,U							* get offset 24 as input (next entry to use)
		 CLRB      							* first directory sector
MOR01  SUBA  #10							* subtract 10 entries (1 sector)
		 BMI   MOR02						* if negative value, adapt result
		 INCB									* increment sector number
		 BRA   MOR01						* keep on subtracting
MOR02	 ADDA  #10							* to get a positive value = entry number where the extents begin	 
       LDU   #EXTENT						* point to beginning of read data area for extents
		 LDX   <SEC1ST						* get first directory sector
		 ABX									* add calculated sector
		 LDB   #25							* entry length
		 MUL									* space to jump over (A was entry to find in the calculated sector)
		 LEAU  D,U							* U points to first byte of desired extents entry (in buffer area)
		 LDY   #EXTENT						* point to extent entries area in RAM
       JSR   (DWDVEC)					* read directory sector
       BCS   MFAIL						* detect frame error
       BNE   MFAIL						* detect incomplete read operation
		 LDA   ,U							* get entry flags (fisrt byte of entry)										
       BITA	 #1							* Flags (offset 0) AND 1 must be equal to 1 (extent entry)
       LBEQ  MSG06						* else we have a problem and must quit program
                              		* For each of the 7 extents in this entry
       LDB   #3								* offset to number of sectors for 1st extent
       LDX   <TABPTR							* get Table pointer back
MOR03  LDA   B,U 								* is number of sectors = 0?
       BEQ   MOR05							* yes, return
		 SUBB  #2								* offset to 1st LSN
 		 LDY   B,U								* get the 1st LSN of the extent
    	 LEAY  1,Y								* because header occupies a whole sector in VDK!		 
 		 STY   ,X++								* put int table
 		 STA   ,X+								* add to table
MOR04	 ADDB  #2+3								* point to next extent - 2 bytes to compensate prior SUBB (step 3 bytes)
		 CMPB  #22								* out of range?
		 BLO   MOR03						* no, process next extent
 		 LDA   ,U							* get entry flags
 		 BITA  #32							* entry flags AND 32 = 32? 
		 BNE   MOREXT						* yes, there are more extents, go back to process them
MOR05	 RTS						   		* return
MFAIL  LEAS  2,S  						* clean stack
 	 	 ANDCC #EIMASK						* enable interrupts
		 JMP   IOERR						* I/O error message
	 
***************************************************************************************************************************
LDTEXT FCC   /LOAD A FILE / 
       FCC   /FROM AN VDK/,13,0		* intro message to the user
NOTVDK FCC   /NOT AN VDK FILE/,0		* bad VDK file
NOTDW  FCC   /VDK NOT FOR DW/,0		* VDK not adapted for DW
BADVDK FCC   /BAD VDK /
		 FCC   /DIRECTORY/,0  			* Bad directory entry
NULFIL FCC   /NULL FILE/,0          * 0 bytes length file
NOTFIL FCC   /NO FILE ENTERED/,0    * No File entered
NDWEEB FCC   /"TABLDR"/					* next dweeb to be called
*
DWRVEC FDB   0								* indirect jump to drivewire read function
DWWVEC FDB   0								* indirect jump to drivewire write function 
DWDVEC FDB   0								* indirect jump to drivewire doread function
* 
